#ifndef __DS485N_PACKET_H__
#define __DS485N_PACKET_H__

/*
 * WARNING
 * This file is processed by SWIG in dsm-api.
 * Always rebuild dsm-api after changing this file
 * Some problems occur only during runtime, verify using
 * https://git.digitalstrom.org/dsmduo/dsmduo-tools/blob/master/ds.lua
 */

#ifndef SWIG
/*  SWIG bombs on __attribute__((packed)) */
#define ATTRIBUTE_PACKED __attribute__((packed))
#else
#define ATTRIBUTE_PACKED
#endif

#include <digitalSTROM/dsuid.h>

#define DS485_MAX_PAYLOAD_SIZE_RAW  128
#define DS485_MAX_PAYLOAD_SIZE      (DS485_MAX_PAYLOAD_SIZE_RAW - 1)

#define DS485_CONTAINER_HDR_LEN     (offsetof(ds485_container_t, data))
#define DS485N_CONT_LEN(cont)       (DS485_CONTAINER_HDR_LEN + (cont)->length)

/**
 * Typ des dS485Packets
 */
typedef enum {
    DS485_CONTAINER_REQUEST = 1,
    DS485_CONTAINER_RESPONSE,
    DS485_CONTAINER_EVENT
} container_type_t;

typedef enum {
    DS485_FLAG_NONE = 1,
    DS485_FLAG_BROADCAST = 2,
} container_flags_t;

/**
 * dS485 API Daten Container
 */
typedef struct ds485_container {
    dsuid_t destinationId;
    dsuid_t sourceId;
    uint32_t containerType;
    uint32_t containerFlags;
    uint8_t length; //< add DS485_CONTAINER_HDR_LEN for package size
    union {
        struct {
            unsigned char transactionId;
            unsigned char data[DS485_MAX_PAYLOAD_SIZE];
        };
        unsigned char raw_data[DS485_MAX_PAYLOAD_SIZE_RAW];
    };
} ATTRIBUTE_PACKED ds485_container_t;

#define DS485N_PACKET_MAX_PAYLOAD_SIZE_RAW  8600
#define DS485N_PACKET_MAX_PAYLOAD_SIZE      DS485N_PACKET_MAX_PAYLOAD_SIZE_RAW - 1

#define DS485N_PACKET_HDR_LEN               (offsetof(ds485n_packet_t, data))
#define DS485N_PKG_LEN(pkg)                 (DS485N_PACKET_HDR_LEN + (pkg)->length)

typedef struct ds485n_packet {
    dsuid_t destinationId;
    dsuid_t sourceId;
    uint16_t dummy; // explictly needed now as we pack the struct; without packing compilers did 32-bit align here
    uint32_t containerType; /* legacy */ // using uint32_t, enum type is dangerous because it could be 1,2 or 4 bytes depending on compiler
    uint16_t length; // TODO: this is only works on a little-endian architecture (x86, most ARM). Should be made endian-independent
    union {
        struct {
            unsigned char transactionId;
            unsigned char data[DS485N_PACKET_MAX_PAYLOAD_SIZE];
        } ATTRIBUTE_PACKED;
        unsigned char raw_data[DS485N_PACKET_MAX_PAYLOAD_SIZE_RAW];
    } ATTRIBUTE_PACKED;
} ATTRIBUTE_PACKED ds485n_packet_t;

#endif /* __DS485N_PACKET_H__ */
