/*
 * This file is part of the digitalSTROM ds485-client library.
 *
 * Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland
 *
 * digitalSTROM ds485-client library is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * digitalSTROM ds485-client library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with digitalSTROM ds485-client library. If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "config.h"
#include "include/ds485-socket-server.h"

#include <errno.h>
#include <pthread.h>
#include <stdlib.h>
#include <string.h>

#include "base/log.h"
#include "ds485-socket-server-impl.h"

NETLIB_EXPORTED
struct dS485_server* ds485_server_init(struct dS485_server_callbacks *cb,
                                       bool upperPath)
{
    struct dS485_server *tmp;
    const char *env;
    int id, loglevel;

    init_logger("ds485n", LOG_NOTICE, 0);

    env = getenv(DS485_NETLIB_DEBUG);
    if (env) {
        errno = 0;
        loglevel = strtol(env, NULL, 10);
        if (errno) {
            report(LOG_WARNING, "Invalid DS485_NETLIB_DEBUG=%s, ingoring\n",
                   env);
        } else {
            set_log_level(loglevel);
        }
    }

    env = getenv(DS485_SYSLOG);
    if (env) {
        enable_syslog(true);
    }

    tmp = calloc(1, sizeof(struct dS485_server));
    if (!tmp) {
        return NULL;
    }

    tmp->active = true;
    tmp->sfd = -1;
    tmp->upperPath = upperPath;
    memcpy(&tmp->cb, cb, sizeof(struct dS485_server_callbacks));

    /*
     * reserve id '0', it could be mistaken with a packet
     * that did not initialize that field
     */
    pthread_rwlock_init(&tmp->clientListRWLock, NULL);
    for (id = 1; id < MAX_CLIENTS; id++) {
        tmp->transid_pool[id] = id;
    }

    return tmp;
}

NETLIB_EXPORTED void ds485_server_cleanup(struct dS485_server *server)
{
    report(LOG_NOTICE, "ds485_server_cleanup\n");
    pthread_rwlock_destroy(&server->clientListRWLock);
    free(server);
}
